@@ -1,3 +1,4 @@
+from testcase.app.disable_module import DisableModule
 from testcase.app.surface_mse_module import SurfaceMSE
 from worker import App
 
@@ -7,5 +8,6 @@ class DrillingEfficiency(App):
     # override
     def get_modules(self):
         return [
-            SurfaceMSE
+            SurfaceMSE,
+            DisableModule
         ]
