@@ -1,4 +1,8 @@
-from worker import constants
+import worker
+import importlib
+importlib.reload(worker)
+from worker import constants  # noqa: #402
+
 
 constants.update({
     "global": {
