@@ -122,9 +122,16 @@ class TestActivity(unittest.TestCase):
         self.assertEqual(1, len(activity_groups))
         self.assertEqual(18, activity_groups[0].duration)
 
+    def test_activity_group_overlap(self):
+        group1 = ActivityGroup(Activity.WASHING_DOWN, start=1, end=10)
+        group2 = ActivityGroup(Activity.WASHING_DOWN, start=5, end=15)
+        group3 = ActivityGroup(Activity.WASHING_DOWN, start=20, end=23)
+
+        self.assertTrue(ActivityGroup.has_overlap(group1, group2))
+        self.assertFalse(ActivityGroup.has_overlap(group2, group3))
+
 
 def get_data_for_activity_grouping():
-    # data from: env=production, asset_id=68483965
     wits_chunk = os.path.abspath(
         os.path.join(os.path.dirname(__file__), '..', 'resources', 'test', 'activity_grouping.data')
     )
