@@ -148,6 +148,28 @@ class App(StateMixin, LoggingMixin, RollbarMixin):
         event = [record for record in event if record.get("collection") in valid_stream_collections]
         return event
 
+    def filter_event_for_duplicates(self, event):
+        last_processed_timestamp = self.state.get('last_processed_timestamp') or 0
+
+        # If event is a single record and greater than the last processed timestamp, return the event
+        if len(event) == 1 and event[0].get("timestamp") > last_processed_timestamp:
+            return event
+
+        # If length of unique timestamps is same as length of event and the first record timestamp is also greater than
+        # last_processed_timestamp, then return the event
+        unique_timestamps = set([record.get("timestamp") for record in event])
+        if len(unique_timestamps) == len(event) and event[0].get("timestamp") > last_processed_timestamp:
+            return event
+
+        # Filtering the events for duplicates, once we identify that duplicates exist.
+        filtered_events = []
+        for each_record in event:
+            if each_record.get("timestamp") > last_processed_timestamp:
+                filtered_events.append(each_record)
+                last_processed_timestamp = each_record.get("timestamp")
+
+        return filtered_events
+
     def load_scheduler_event(self, asset_id, event, max_lookback_seconds):
         """
         To load a scheduler event and get the wits stream data
@@ -169,8 +191,7 @@ class App(StateMixin, LoggingMixin, RollbarMixin):
 
         return scheduler_event
 
-    @staticmethod
-    def load_wits_stream_event(asset_id, event, max_lookback_seconds):
+    def load_wits_stream_event(self, asset_id, event, max_lookback_seconds):
         """
         To load a wits stream event and get more data if necessary
         :param asset_id: The asset to load
@@ -180,6 +201,9 @@ class App(StateMixin, LoggingMixin, RollbarMixin):
         """
 
         if event and max_lookback_seconds:
+            # First filtering original event for duplicates
+            event = self.filter_event_for_duplicates(event)
+
             first_timestamp = event[0].get('timestamp')
 
             if not first_timestamp:
