@@ -1,7 +1,7 @@
-from worker.app.modules.__init__ import Module
+from worker.app.modules.activity_module import ActivityModule
 
 
-class TimeActivityModule(Module):
+class TimeActivityModule(ActivityModule):
     """
     This is an abstract time-base module that needs to be extended by actual modules.
     An example of this module would be running an app every 1 minute.
