@@ -145,21 +145,22 @@ def setup_redis_worker(env: str) -> Tuple:
     return redis_worker
 
 
-def get_drillstring_by_id(drillstring_id: str) -> Union[dict, None]:
+def get_config_by_id(string_id: str, collection: str) -> [dict, None]:
     """
-    Get the drillstring from API by providing mongodb _id
-    :param drillstring_id: mongodb _id of the drillstring
+    Get the drillstring or casingstring from API by providing mongodb _id
+    :param string_id: mongodb _id of the drillstring
+    :param collection:
     :return:
     """
-    drillstring = None
+    string = None
     try:
-        drillstring = API().get_by_id(
-            path="/v1/data/corva/", collection='data.drillstring', id=drillstring_id
+        string = API().get_by_id(
+            path="/v1/data/corva/", collection=collection, id=string_id
         ).data
     except NotFound:
         pass
 
-    return drillstring
+    return string
 
 
 def is_number(data):
@@ -430,3 +431,28 @@ def compute_time_step(records: Union[List[WITS], List[dict], List[float]], perce
         return None
 
     return int(time_step)
+
+
+def compare_float(value1, value2, tolerance):
+    """
+    Compare two values based on the given tolerance
+    :param value1: first number
+    :param value2: second number
+    :param tolerance: tolerance of the comparison
+    :return: +1 when value1 > value2 + tolerance, -1 when value2 > value1 + tolerance, 0 otherwise
+    """
+    if value1 > value2 + tolerance:
+        return 1
+
+    if value2 > value1 + tolerance:
+        return -1
+
+    return 0
+
+
+def is_stream_app():
+    lambda_name = os.getenv("AWS_LAMBDA_FUNCTION_NAME")
+    if not lambda_name:
+        return False
+
+    return "task" not in lambda_name.lower()
