@@ -3,7 +3,7 @@ from typing import Union, List
 
 from worker.data.operations import get_data_by_path
 from worker.data.serialization import serialization
-from worker.wellbore.model.drillstring_components import Pipe, Bit, Agitator, PDM, MWD, RSS
+from worker.wellbore.model.drillstring_components import Pipe, Bit, Agitator, PDM, MWD, RSS, UnderReamer
 from worker.wellbore.model.hole import Hole
 from worker.wellbore.sections_mixin import SectionsMixin
 from worker.wellbore.model.enums import PipeType
@@ -13,11 +13,13 @@ from worker.wellbore.model.enums import PipeType
 class Drillstring(SectionsMixin):
     SERIALIZED_VARIABLES = {
         'sections': list,
+        'mongo_id': str
     }
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
+        self.mongo_id = None
         self.sections: List[Union[Pipe, Bit]] = kwargs.get('sections') or []
         if args:
             self.set_drillstring(args[0])
@@ -92,6 +94,8 @@ class Drillstring(SectionsMixin):
         if not drillstring:
             return
 
+        self.mongo_id = drillstring.get("_id")
+
         self.sections = []
 
         # family to class mapping
@@ -100,7 +104,8 @@ class Drillstring(SectionsMixin):
             'agitator': Agitator,
             'pdm': PDM,
             'mwd': MWD,
-            'rss': RSS
+            'rss': RSS,
+            'ur': UnderReamer
         }
 
         drillstring_components = drillstring.get("data", {}).get("components", [])
@@ -181,6 +186,16 @@ class Drillstring(SectionsMixin):
 
         return None
 
+    def get_under_reamer(self):
+        """
+        Get the under-reamer element from the drillstring
+        :return:
+        """
+        if not self:
+            return None
+
+        return next((pipe for pipe in self if pipe.pipe_type == PipeType.UNDERREAMER), None)
+
     def get_bha_length(self) -> float:
         """
         To get the length of the BHA
