@@ -1,12 +1,16 @@
+import time
 from copy import deepcopy
+from enum import Enum
 
 import numpy as np
 
+from worker.data.alert import Alerter
 from worker.data.serialization import serialization
 from worker.data.unit_conversions import US_LIQUID_GAL_to_INCH3
+from worker.mixins.logging import Logger
 from worker.wellbore.model.element import Element
-from worker.wellbore.model.enums import PipeType, PipeMaterial
-from worker.data.operations import nanround, equal
+from worker.wellbore.model.enums import PipeType, PipeMaterial, HoleType
+from worker.data.operations import nanround, equal, is_stream_app
 
 
 @serialization
@@ -308,3 +312,132 @@ class Bit(Element):
             f"tfa={nanround(self.tfa, 2)} in^2, "
             f"{super().__repr__()}"
         )
+
+
+@serialization
+class UnderReamer(Pipe):
+    SERIALIZED_VARIABLES = deepcopy(Pipe.SERIALIZED_VARIABLES)
+    SERIALIZED_VARIABLES.update({
+        'activation_logic': str,
+        'ur_opened_od': float,
+        'ur_opened_depth': float,
+
+        'flow_rate': float,
+        '_is_opened': bool,
+        'tfa': float,
+        'alerted_at_timestamp': float,
+    })
+
+    class UnderReamerActivationType(Enum):
+        ALREADY_ACTIVATED = "already_activated"
+        MUD_FLOW_RATE = "flowrate_activated"
+        OPEN_HOLE = "open_hole_activated"
+        UNACTIVATED = "did_not_activate"
+
+    @serialization
+    class ActivationLogic(Enum):
+        OPEN_HOLE_ACTIVATED = "open_hole_activated"
+        FLOW_ACTIVATED = "flow_activated"
+
+    def __init__(self, **kwargs):
+        super(UnderReamer, self).__init__(**kwargs)
+
+        self._is_opened: bool = False
+        self.alerted_at_timestamp: float = 0.0
+        self.ALERT_BACKOFF_TIME = 3600
+
+        self.tfa = float(kwargs['tfa'])
+        self.ur_opened_depth = kwargs.get('ur_opened_depth')
+        self.ur_opened_od = float(kwargs['ur_opened_od'])
+        self.flow_rate = float(kwargs.get('flow_rate'))
+        self.activation_logic = self.ActivationLogic(kwargs['activation_logic'])
+        self.pipe_type = PipeType.UNDERREAMER
+
+    @property
+    def discharge_coefficient(self) -> float:
+        return 0.95
+
+    @property
+    def is_opened(self) -> bool:
+        """
+        This property is needed for enabling split flow
+        :return:
+        """
+        return self._is_opened and self.tfa > 0.0
+
+    def set_opened(self, opened: bool):
+        self._is_opened = opened
+
+    def get_activation_type(self, mud_flow_rate: float):
+        opened_depth = self.ur_opened_depth or 10_000_000.0
+
+        # if it was activated before no need to check how it was activated just use `ur_opened_depth` key
+        if self.top_depth >= opened_depth:
+            return self.UnderReamerActivationType.ALREADY_ACTIVATED
+
+        # if it was never activated and logic is open hole
+        if self.activation_logic == self.ActivationLogic.OPEN_HOLE_ACTIVATED and not self.ur_opened_depth:
+            return self.UnderReamerActivationType.OPEN_HOLE
+
+        # if it was never activated and logic is flow activated
+        if mud_flow_rate > self.flow_rate and not self.ur_opened_depth:
+            return self.UnderReamerActivationType.MUD_FLOW_RATE
+
+        # Otherwise unactivated
+        return self.UnderReamerActivationType.UNACTIVATED
+
+    def activate_under_reamer(self, mud_flow_rate, start_reaming_depth, hole_type: HoleType):
+        activation_mode = self.get_activation_type(mud_flow_rate)
+
+        if activation_mode == self.UnderReamerActivationType.UNACTIVATED:
+            self.ur_opened_depth = None
+            return activation_mode
+
+        # If OpenHole activated then set the opened depth to lastCasingDepth
+        if activation_mode == self.UnderReamerActivationType.OPEN_HOLE:
+            if start_reaming_depth is None:
+                Logger.warn("Unable to find the last casing depth")
+                self.ur_opened_depth = None
+                return self.UnderReamerActivationType.UNACTIVATED
+
+            # No need to raise Alert here
+            if start_reaming_depth >= self.top_depth:
+                self.ur_opened_depth = None
+                return self.UnderReamerActivationType.UNACTIVATED
+
+            self.ur_opened_depth = start_reaming_depth
+            return activation_mode
+
+        # If mud activated then set the opened depth as top depth of UR
+        if self.ur_opened_depth is None:
+            self.ur_opened_depth = self.top_depth
+
+        # Check if under-reamer was opened in cased hole
+        if self.ur_opened_depth <= start_reaming_depth and hole_type == HoleType.CASED_HOLE:
+            self.trigger_alert(self.ur_opened_depth, start_reaming_depth)
+            self.ur_opened_depth = None
+            return self.UnderReamerActivationType.UNACTIVATED
+
+        return activation_mode
+
+    def trigger_alert(self, ur_opened_depth, start_reaming_depth):
+        identifier = "ur_opened_in_cased_hole"
+        message = f"Attempted to open under-reamer inside cased hole at {ur_opened_depth}. " \
+                  f"Last casing shoe depth of {start_reaming_depth}. "
+
+        current_timestamp = time.time()
+
+        if (current_timestamp > self.alerted_at_timestamp + self.ALERT_BACKOFF_TIME) and is_stream_app():
+            Alerter.trigger_alert(identifier, message)
+            self.alerted_at_timestamp = current_timestamp
+            Logger.warn(f"Alert created. identifier: {identifier}, message: {message}")
+        else:
+            Logger.info(f"Alert created. identifier: {identifier}, message: {message}"
+                        f"However, not triggering alert because it was not triggered from Stream app")
+
+    def __repr__(self):
+        return (
+            f"{super().__repr__()}"
+            f"opened_od={nanround(self.ur_opened_od, 2)} in - "
+            f"tfa={nanround(self.tfa, 2)} in^2, "
+        )
